@@ -30,6 +30,7 @@
 #include "defaultdevice.h"
 #include "indiguiderinterface.h"
 #include "indipropertynumber.h"
+#include "indipropertyswitch.h"
 #include "inditimer.h"
 #include "indielapsedtimer.h"
 #include "dsp/manager.h"
@@ -41,7 +42,7 @@
 
 #include <fitsio.h>
 
-#include <memory>
+#include <map>
 #include <cstring>
 #include <chrono>
 #include <stdint.h>
@@ -132,6 +133,15 @@ class CCD : public DefaultDevice, GuiderInterface
 
         typedef enum { UPLOAD_CLIENT, UPLOAD_LOCAL, UPLOAD_BOTH } CCD_UPLOAD_MODE;
 
+        typedef struct CaptureFormat
+        {
+            std::string name;
+            std::string label;
+            uint8_t bitsPerPixel {8};
+            bool isDefault {false};
+            bool isLittleEndian {true};
+        } CaptureFormat;
+
         virtual bool initProperties() override;
         virtual bool updateProperties() override;
         virtual void ISGetProperties(const char * dev) override;
@@ -467,6 +477,13 @@ class CCD : public DefaultDevice, GuiderInterface
          */
         virtual bool StopStreaming();
 
+        /**
+         * @brief SetCaptureFormat Set Active Capture format.
+         * @param index Index of capture format from CaptureFormatSP property.
+         * @return True if change is successful, false otherwise.
+         */
+        virtual bool SetCaptureFormat(uint8_t index);
+
         /**
          * \brief Add FITS keywords to a fits file
          * \param fptr pointer to a valid FITS file.
@@ -520,6 +537,23 @@ class CCD : public DefaultDevice, GuiderInterface
          */
         virtual void checkTemperatureTarget();
 
+        /**
+         * @brief processFastExposure After an exposure is complete, check if fast
+         * exposure was enabled. If it is, then immediately start the next exposure
+         * if possible and decrement the counter.
+         * @param targetChip Active fast exposure chip.
+         * @return True if next fast exposure is started, false otherwise.
+         */
+        virtual bool processFastExposure(CCDChip * targetChip);
+
+        /**
+         * @brief addCaptureFormat Add a supported camera native capture format (e.g. Mono, Bayer8..etc)
+         * @param name Name of format (e.g. FORMAT_MONO)
+         * @param label Label of format (e.g. Mono)
+         * @param isDefault If true, it would be set as the default format if there is config file does not specify one.
+         * Config file override default format.
+         */
+        virtual void addCaptureFormat(const CaptureFormat &format);
 
         // Epoch Position
         double RA, Dec;
@@ -530,6 +564,11 @@ class CCD : public DefaultDevice, GuiderInterface
         // J2000 Position
         double J2000RA;
         double J2000DE;
+        bool J2000Valid;
+
+        // exposure information
+        char exposureStartTime[MAXINDINAME];
+        double exposureDuration;
 
         double primaryFocalLength, primaryAperture, guiderFocalLength, guiderAperture;
         bool InExposure;
@@ -575,6 +614,8 @@ class CCD : public DefaultDevice, GuiderInterface
         std::vector<std::string> FilterNames;
         int CurrentFilterSlot {-1};
 
+        std::vector<CaptureFormat> m_CaptureFormats;
+
         std::unique_ptr<StreamManager> Streamer;
         std::unique_ptr<DSP::Manager> DSP;
         CCDChip PrimaryCCD;
@@ -593,6 +634,14 @@ class CCD : public DefaultDevice, GuiderInterface
         INumberVectorProperty EqNP;
         INumber EqN[2];
 
+        /**
+         * @brief J200EqNP Snoop property to read the equatorial J2000 coordinates of the mount.
+         * ActiveDeviceTP defines snoop devices and the driver listens to this property emitted
+         * by the mount driver if specified. It is important to generate a proper FITS header.
+         */
+        INumberVectorProperty J2000EqNP;
+        INumber J2000EqN[2];
+
         /**
          * @brief ActiveDeviceTP defines 4 devices the camera driver can listen to (snoop) for
          * properties of interest so that it can generate a proper FITS header.
@@ -644,6 +693,17 @@ class CCD : public DefaultDevice, GuiderInterface
         ITextVectorProperty FileNameTP;
         IText FileNameT[1] {};
 
+        /// Specifies Camera NATIVE capture format (e.g. Mono, RGB, RAW8..etc).
+        INDI::PropertySwitch CaptureFormatSP {0};
+
+        /// Specifies Driver image encoding format (FITS, Native, JPG, ..etc)
+        INDI::PropertySwitch EncodeFormatSP {2};
+        enum
+        {
+            FORMAT_FITS,     /*!< Save Image as FITS format  */
+            FORMAT_NATIVE    /*!< Save Image as the native format of the camera itself. */
+        };
+
         ISwitch UploadS[3];
         ISwitchVectorProperty UploadSP;
 
@@ -711,7 +771,9 @@ class CCD : public DefaultDevice, GuiderInterface
     private:
         uint32_t capability;
 
-        bool m_ValidCCDRotation;
+        bool m_ValidCCDRotation {false};
+        std::string m_ConfigCaptureFormatLabel;
+        int m_ConfigEncodeFormatIndex {-1};
         int m_ConfigFastExposureIndex {INDI_DISABLED};
 
         ///////////////////////////////////////////////////////////////////////////////
